# Generated by Django 3.2 on 2021-04-20 17:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mail_office', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='emailmessage',
            name='template',
        ),
        migrations.AlterField(
            model_name='emailmessage',
            name='status',
            field=models.CharField(choices=[('new', 'New'), ('in_sending', 'In Sending'), ('canceled', 'Canceled'), ('failed', 'Failed'), ('sent', 'Sent')], default='new', max_length=10),
        ),
        migrations.DeleteModel(
            name='EmailMessageTemplate',
        ),
    ]
